from evosnap import constants


class CardData:
    def __init__(self, card_type, p_a_n, cardholder_name=None, expire=None, track1_data=None, track2_data=None):
        """
        Contains information about the payment card. Conditional, required for Authorize and AuthorizeAndCapture
        transactions. May be required for undoing PIN Debit transactions.
        :param activation: Contains information for activating an account. This is a required element.
        """
        self.__camelcase=constants.ALL_FIELDS
        self.__order=['CardType', 'PAN', 'CardholderName', 'Expire', 'Track1Data', 'Track2Data']
        self.cardholder_name=cardholder_name
        self.card_type=card_type
        self.expire=expire
        self.p_a_n=p_a_n
        self.track1_data=track1_data
        self.track2_data=track2_data



from evosnap import constants


class TransactionTenderData:
    def __init__(self, encryption_key_id=None, payment_account_data_token=None, secure_payment_account_data=None,
                 swipe_status=None, card_data=None, card_security_data=None, ecommerce_security_data=None):
        """
        Base class object containing information about the tender used for a specific transaction.
        This element is Required.
        :param activation: Contains information for activating an account. This is a required element.
        """
        self.__camelcase=constants.ALL_FIELDS
        self.__order=['EncryptionKeyId', 'PaymentAccountDataToken', 'SecurePaymentAccountData',
                      'SwipeStatus', 'CardData', 'CardSecurityData', 'EcommerceSecurityData']
        self.encryption_key_id=encryption_key_id
        self.payment_account_data_token=payment_account_data_token
        self.secure_payment_account_data=secure_payment_account_data
        self.swipe_status=swipe_status
        self.card_data=card_data
        self.card_security_data=card_security_data
        self.ecommerce_security_data=ecommerce_security_data



import json
from collections import OrderedDict

# Compat between Python 3.4 and Python 3.5
if not hasattr(json, 'JSONDecodeError'):
    json.JSONDecodeError = ValueError
from json import JSONDecodeError

import requests

from evosnap.data_element_encoder import DataElementEncoder
from evosnap.exceptions import TransactionRequestException
from evosnap.response import Response
from evosnap.service_information.mixins import ServiceInformationRequestMixin
from evosnap.transactions.enum import RequestType


class TransactionRequestMixin(ServiceInformationRequestMixin):
    def __init__(self, **kwargs):
        """
        Basic information about the check being processed. This element is Required.
        :param activation: Contains information for activating an account. This is a required element.
        """
        self.cws_tps_baseurl=kwargs.get('cws_tps_baseurl')

        self.data = kwargs.get('data', '')
        self.workflow_id=kwargs.get('workflow_id', '')
        self.batch_ids=kwargs.get('batch_ids')
        self.transaction_ids=kwargs.get('transaction_ids')
        self.force_close=kwargs.get('force_close')
        super(TransactionRequestMixin, self).__init__(**kwargs)

    @property
    def __authorize_json(self):
        return json.dumps(OrderedDict([
            ('$type', RequestType.authorize.value),
            ('ApplicationProfileId', self.application_profile_id),
            ('MerchantProfileId', self.merchant_profile_id),
            ('Transaction', self.data),
        ]), cls=DataElementEncoder)

    @property
    def __authorize_and_capture_json(self):
        return json.dumps(OrderedDict([
            ('$type', RequestType.authorize_and_capture.value),
            ('ApplicationProfileId', self.application_profile_id),
            ('MerchantProfileId', self.merchant_profile_id),
            ('Transaction', self.data),
        ]), cls=DataElementEncoder)

    @property
    def __manage_account_json(self):
        return json.dumps(OrderedDict([
            ('$type', RequestType.manage_account.value),
            ('ApplicationProfileId', self.application_profile_id),
            ('MerchantProfileId', self.merchant_profile_id),
            ('Transaction', self.data),
        ]), cls=DataElementEncoder)

    @property
    def __adjust_json(self):
        return json.dumps(OrderedDict([
            ('$type', RequestType.adjust.value),
            ('ApplicationProfileId', self.application_profile_id),
            ('DifferenceData', self.data),
        ]), cls=DataElementEncoder)

    @property
    def __undo_json(self):
        return json.dumps(OrderedDict([
            ('$type', RequestType.undo.value),
            # ('sessionToken', None),
            ('differenceData', self.data),
            ('applicationProfileId', self.application_profile_id),
        ]), cls=DataElementEncoder)

    @property
    def __capture_json(self):
        return json.dumps(OrderedDict([
            ('$type', RequestType.capture.value),
            ('ApplicationProfileId', self.application_profile_id),
            ('DifferenceData', self.data),
        ]), cls=DataElementEncoder)

    @property
    def __resubmit_json(self):
        return json.dumps(OrderedDict([
            ('$type', RequestType.resubmit.value),
            ('ApplicationProfileId', self.application_profile_id),
            ('MerchantProfileId', self.merchant_profile_id),
            ('Transaction', self.data),
        ]), cls=DataElementEncoder)

    @property
    def __capture_selective_json(self):
        return json.dumps(OrderedDict([
            ('$type', RequestType.capture_selective.value),
            ('ApplicationProfileId', self.application_profile_id),
            ('DifferenceData', self.data),
            ('TransactionIds', self.transaction_ids),
        ]), cls=DataElementEncoder)

    @property
    def __capture_all_json(self):
        return json.dumps(OrderedDict([
            ('$type', RequestType.capture_all.value),
            ('ApplicationProfileId', self.application_profile_id),
            ('MerchantProfileId', self.merchant_profile_id),
            ('DifferenceData', self.data),
            ('BatchIds', self.batch_ids),
            ('ForceClose', self.force_close)
        ]), cls=DataElementEncoder)

    @property
    def __return_by_id_json(self):
        return json.dumps(OrderedDict([
            ('$type', RequestType.return_by_id.value),
            ('ApplicationProfileId', self.application_profile_id),
            ('MerchantProfileId', self.merchant_profile_id),
            ('DifferenceData', self.data),
        ]), cls=DataElementEncoder)

    @property
    def __return_unlinked_json(self):
        return json.dumps(OrderedDict([
            ('$type', RequestType.return_unlinked.value),
            ('ApplicationProfileId', self.application_profile_id),
            ('MerchantProfileId', self.merchant_profile_id),
            ('Transaction', self.data),
        ]), cls=DataElementEncoder)
        # return json.dumps(OrderedDict([
        #     ('$type', RequestType.return_unlinked.value),
        #     # ('ApplicationProfileId', self.application_profile_id),
        #     # ('MerchantProfileId', self.merchant_profile_id),
        #     # ('Transaction', self.data),
        # ]), cls=DataElementEncoder)

    @property
    def request_json(self):
        if self.request_type == RequestType.authorize:
            return self.__authorize_json
        elif self.request_type == RequestType.authorize_and_capture:
            return self.__authorize_and_capture_json
        elif self.request_type == RequestType.manage_account:
            return self.__manage_account_json
        elif self.request_type == RequestType.adjust:
            return self.__adjust_json
        elif self.request_type == RequestType.undo:
            return self.__undo_json
        elif self.request_type == RequestType.capture:
            return self.__capture_json
        elif self.request_type == RequestType.resubmit:
            return self.__resubmit_json
        elif self.request_type == RequestType.capture_selective:
            return self.__capture_selective_json
        elif self.request_type == RequestType.capture_all:
            return self.__capture_all_json
        elif self.request_type == RequestType.return_by_id:
            return self.__return_by_id_json
        elif self.request_type == RequestType.return_unlinked:
            return self.__return_unlinked_json

    @property
    def request_endpoint(self):
        if self.request_type == RequestType.authorize:
            return self.cws_tps_baseurl+'/'+self.workflow_id
        elif self.request_type == RequestType.authorize_and_capture:
            return self.cws_tps_baseurl+'/'+self.workflow_id
        elif self.request_type == RequestType.manage_account:
            return self.cws_tps_baseurl+'/'+self.workflow_id
        elif self.request_type == RequestType.adjust:
            return self.cws_tps_baseurl+'/'+self.workflow_id+'/'+self.data.transaction_id
        elif self.request_type == RequestType.undo:
            return self.cws_tps_baseurl+'/'+self.workflow_id+'/'+self.data.transaction_id
        elif self.request_type == RequestType.capture:
            return self.cws_tps_baseurl+'/'+self.workflow_id+'/'+self.data.transaction_id
        elif self.request_type == RequestType.resubmit:
            return self.cws_tps_baseurl+'/'+self.workflow_id
        elif self.request_type == RequestType.capture_selective:
            return self.cws_tps_baseurl+'/'+self.workflow_id
        elif self.request_type == RequestType.capture_all:
            return self.cws_tps_baseurl+'/'+self.workflow_id
        elif self.request_type == RequestType.return_by_id:
            return self.cws_tps_baseurl+'/'+self.workflow_id
        elif self.request_type == RequestType.return_unlinked:
            return self.cws_tps_baseurl+'/'+self.workflow_id

    @property
    def request_method(self):
        if self.request_type in [RequestType.adjust, RequestType.undo, RequestType.capture]:
            return 'PUT'
        else:
            return 'POST'

    def send(self, test_request=None):
        self.sign_on()
        # print(self.request_endpoint)
        response = requests.request(
            method=self.request_method,
            url=self.request_endpoint,
            headers={'Content-Type': 'application/json', 'Accept': 'application/json'},
            data=self.request_json if not test_request else json.dumps(test_request),
            auth=(self.session_token, ''),
            verify=self.ssl_verification
        )
        if response.status_code!=200 and response.status_code!=201 and response.status_code!=400:
            try:
                msg = str(response.status_code)+' '+Response(response.text).to_pretty_json()
                # print(response.request.body)
                # print(response.headers)
                # print(response.request.headers)
                raise TransactionRequestException(msg)
            except JSONDecodeError:
                msg = str(response.status_code) + ' ' + response.text
                # print(response.headers)
                # print(response.headers)
                raise TransactionRequestException(msg)
        return Response(response.text)



from evosnap import constants


class TransactionCustomerData:
    def __init__(self, billing_data=None, customer_id=None, customer_tax_id=None,
                 shipping_data=None):
        """
        Contains information about the customer. This element is required by some service providers. If this value is
        set on the transaction and not required by the service provider, the data is not present on the transaction.
        :param activation: Contains information for activating an account. This is a required element.
        """
        self.__camelcase=constants.ALL_FIELDS
        self.__order=['BillingData', 'CustomerId', 'CustomerTaxId',
                      'ShippingData']
        self.billing_data=billing_data
        self.customer_id=customer_id
        self.customer_tax_id=customer_tax_id
        self.shipping_data=shipping_data



from evosnap import constants


class Adjust:
    def __init__(self, addendum, amount, tip_amount=None, transaction_id=None):
        """
        Contains address information for either the merchant or customer. This element is Optional.
        :param activation: Contains information for activating an account. This is a required element.
        """
        self.__camelcase=constants.ALL_FIELDS
        self.__order=['Addendum', 'Amount', 'TipAmount', 'TransactionId']
        self.addendum=addendum
        self.amount=amount
        self.tip_amount=tip_amount
        self.transaction_id=transaction_id



from evosnap import constants
from evosnap.transactions.enum import CVDataProvided


class CardSecurityData:
    def __init__(self, international_a_v_s_data=None, c_v_data=None, c_v_data_provided=None, identification_information=None,
                 key_serial_number=None, p_i_n=None, international_a_v_s_override= None):
        """
        Contains security information for the payment card. This element is required for PIN Debit transactions.
        :param activation: Contains information for activating an account. This is a required element.
        """
        self.__camelcase=constants.ALL_FIELDS
        self.__order=['InternationalAVSData', 'InternationalAVSOverride', 'CVData', 'CVDataProvided', 'IdentificationInformation',
                 'KeySerialNumber', 'PIN']
        self.international_a_v_s_data=international_a_v_s_data
        self.international_a_v_s_override=international_a_v_s_override
        self.c_v_data=c_v_data
        if not c_v_data_provided and c_v_data:
            self.c_v_data_provided=CVDataProvided.provided
        elif not c_v_data_provided and not c_v_data:
            self.c_v_data_provided=CVDataProvided.not_set
        else:
            self.c_v_data_provided=c_v_data_provided
        self.identification_information=identification_information
        self.key_serial_number=key_serial_number
        self.p_i_n=p_i_n



from evosnap import constants
from evosnap.transactions.mixins import TransactionRequestMixin


class Request(TransactionRequestMixin):
    def __init__(self, **kwargs):
        """
        Basic information about the check being processed. This element is Required.
        :param activation: Contains information for activating an account. This is a required element.
        """
        super(Request, self).__init__(**kwargs)


from evosnap import constants


class POSDevice:
    def __init__(self,**kwargs):
        self.__order = [
            'posDeviceType', 'posDeviceConnection', 'posDeviceColour', 'posDeviceQuantity',
        ]
        self.__lower_camelcase = constants.ALL_FIELDS

        self.pos_device_type = kwargs.get('pos_device_type')
        self.pos_device_connection = kwargs.get('pos_device_connection')
        self.pos_device_colour = kwargs.get('pos_device_colour')
        self.pos_device_quantity = kwargs.get('pos_device_quantity')

    @property
    def hash_str(self):
        required = [
            'pos_device_type', 'pos_device_connection', 'pos_device_colour', 'pos_device_quantity',
        ]
        return ''.join([str(getattr(self,f)).strip() for f in required if getattr(self,f) is not None])



from evosnap import constants


class Unmanaged:
    def __init__(self, any):
        """
        Can be used to pass service-specific information such as username/password or session token credentials.
        This element is Optional.
        :param activation: Contains information for activating an account. This is a required element.
        """
        self.__camelcase=constants.ALL_FIELDS
        self.any=any



from evosnap import constants


class POSDeploymentLocation:
    def __init__(self,**kwargs):
        self.__order = [
            'posDLTradeName', 'posDLContactFirstName', 'posDLContactLastName', 'posDLAddress',
            'posDLEmail', 'posDLCity', 'posDLCounty', 'posDLPostalCode',
            'posDLCountry', 'posDLPhone', 'posDLRentalTermID',
            'posDLTerminalArray', 'posDLExistingSENumber',
        ]
        self.__lower_camelcase = constants.ALL_FIELDS

        self.pos_d_l_trade_name = kwargs.get('pos_d_l_trade_name')
        self.pos_d_l_contact_first_name = kwargs.get('pos_d_l_contact_first_name')
        self.pos_d_l_contact_last_name = kwargs.get('pos_d_l_contact_last_name')
        self.pos_d_l_address = kwargs.get('pos_d_l_address')
        self.pos_d_l_email = kwargs.get('pos_d_l_email')
        self.pos_d_l_city = kwargs.get('pos_d_l_city')
        self.pos_d_l_county = kwargs.get('pos_d_l_county')
        self.pos_d_l_postal_code = kwargs.get('pos_d_l_postal_code')
        self.pos_d_l_country = kwargs.get('pos_d_l_country')
        self.pos_d_l_phone = kwargs.get('pos_d_l_phone')
        self.pos_d_l_rental_term_i_d = kwargs.get('pos_d_l_rental_term_i_d')
        self.pos_d_l_terminal_array = kwargs.get('pos_d_l_terminal_array')
        self.pos_d_l_existing_s_e_number = kwargs.get('pos_d_l_existing_s_e_number')

    @property
    def pos_d_l_terminal_array_hash(self):
        return ''.join([f.hash_str for f in self.pos_d_l_terminal_array])

    @property
    def hash_str(self):
        required = [
            'pos_d_l_trade_name', 'pos_d_l_contact_first_name', 'pos_d_l_contact_last_name', 'pos_d_l_address',
            'pos_d_l_email', 'pos_d_l_city', 'pos_d_l_county', 'pos_d_l_postal_code',
            'pos_d_l_country', 'pos_d_l_phone', 'pos_d_l_rental_term_i_d',
            'pos_d_l_terminal_array_hash', 'pos_d_l_existing_s_e_number',
        ]
        return ''.join([str(getattr(self,f)).strip() for f in required if getattr(self,f) is not None])


import json
from json import JSONDecodeError
from time import time

import requests

from evosnap import constants
from evosnap.exceptions import SignOnException, AppProfileException
from evosnap.transactions.enum import RequestType

class ServiceInformationRequestMixin:
    def __init__(self, **kwargs):
        """
        Basic information about the check being processed. This element is Required.
        :param activation: Contains information for activating an account. This is a required element.
        """
        self.ssl_verification=kwargs.get('ssl_verification')

        self.cws_sis_baseurl=kwargs.get('cws_sis_baseurl')
        self.cws_tms_baseurl=kwargs.get('cws_tms_baseurl')

        self.identity_token=kwargs.get('identity_token')
        self.session_token=kwargs.get('session_token')
        self.session_token_time=kwargs.get('session_token_time')

        self.service_key=kwargs.get('service_key')
        self.username=kwargs.get('username')
        self.password=kwargs.get('password')
        self.new_password=kwargs.get('new_password')
        self.new_username=kwargs.get('new_username')
        self.new_email=kwargs.get('new_email')

        self.request_type=kwargs.get('request_type')

        self.application_profile_id=kwargs.get('application_profile_id')
        self.service_id=kwargs.get('service_id')
        self.merchant_profile_id=kwargs.get('merchant_profile_id')
        super(ServiceInformationRequestMixin, self).__init__()

    def is_signed_on(self):
        return self.session_token and time()-self.session_token_time <= constants.SESSION_TOKEN_TIME

    def sign_on(self):
        if not self.is_signed_on():
            response = requests.get(
                url=self.cws_sis_baseurl+'/token',
                auth=(self.identity_token,''),
                headers={'Content-Type': 'application/json', 'Accept': 'application/json'},
                verify=self.ssl_verification
            )
            if response.status_code == 200:
                self.session_token_time = time()
                print(response.text)
                self.session_token = json.loads(response.text)
            else:
                raise SignOnException(response.text)

    def app_profile(self):
        self.sign_on()
        response = requests.get(
            url=self.cws_sis_baseurl+'/appProfile/'+self.application_profile_id,
            auth=(self.session_token, ''),
            headers={'Content-Type': 'application/json', 'Accept': 'application/json'},
            verify=self.ssl_verification
        )
        if response.status_code == 200:
            return json.dumps(json.loads(response.text), indent=4, sort_keys=True)
        else:
            try:
                msg = str(response.status_code)+' '+json.dumps(json.loads(response.text), indent=4, sort_keys=True)
                raise AppProfileException(msg)
            except JSONDecodeError:
                raise AppProfileException(str(response.status_code)+' '+response.text)




import enum
import inspect
from collections import OrderedDict
from json import JSONEncoder

import sys

from evosnap import constants


class DataElementEncoder(JSONEncoder):

    @staticmethod
    def in_module(obj):
        module = __name__.split('.')[0]
        classes = [ x[1] for x in inspect.getmembers(
            sys.modules[module],
            lambda member: inspect.isclass(member) and module in member.__module__
        )]
        return any([isinstance(obj, c) for c in classes])

    def default(self, o):
        if self.in_module(o) and not isinstance(o, enum.Enum):
            __camelcase = '_'+o.__class__.__name__+'__camelcase'
            __lower_camelcase = '_'+o.__class__.__name__+'__lower_camelcase'
            __exclude = '_'+o.__class__.__name__+'__exclude'
            __order = '_'+o.__class__.__name__+'__order'
            res = dict(o.__dict__)
            if 'i_type' in res:
                res['$type'] = res.pop('i_type')
            if __exclude in res:
                items = res.pop(__exclude)
                for i in items:
                    res.pop(i)
            if __camelcase in res:
                items = res.pop(__camelcase)
                if items == constants.ALL_FIELDS:
                    items = list(res.keys())
                for i in items:
                    if i != '$type' and i != __order:
                        res[self.camel_case(i)] = res.pop(i)
            if __lower_camelcase in res:
                items = res.pop(__lower_camelcase)
                if items == constants.ALL_FIELDS:
                    items = list(res.keys())
                for i in items:
                    if i != '$type' and i != __order:
                        res[self.lower_camel_case(i)] = res.pop(i)
            if __order in res:
                res = self.order_dict(res, res.get(__order))
            return res
        if isinstance(o, enum.Enum):
            return o.value
        return super(DataElementEncoder, self).default(o)

    @staticmethod
    def camel_case(string):
        return string.replace('_', ' ').title().replace(' ','')

    @staticmethod
    def lower_camel_case(string):
        components = string.split('_')
        return components[0] + "".join(x.title() for x in components[1:])

    def order_dict(self, d, order):
        if order:
            return OrderedDict([(k, d.get(k)) for k in order])
        else:
            return OrderedDict(sorted(d, key=lambda t: t[0]))



from enum import Enum


class RequestType(Enum):
    authorize = 'AuthorizeTransaction, http://schemas.evosnap.com/CWS/v2.0/Transactions/Rest'
    authorize_and_capture = 'AuthorizeAndCaptureTransaction, http://schemas.evosnap.com/CWS/v2.0/Transactions/Rest'
    manage_account = 'ManageAccount, http://schemas.evosnap.com/CWS/v2.0/Transactions/Rest'
    adjust = 'Adjust, http://schemas.evosnap.com/CWS/v2.0/Transactions/Rest'
    undo = 'Undo,http://schemas.evosnap.com/CWS/v2.0/Transactions/Rest'
    capture = 'Capture,http://schemas.evosnap.com/CWS/v2.0/Transactions/Rest'
    resubmit = 'ResubmitTransaction, http://schemas.evosnap.com/CWS/v2.0/Transactions/Rest'
    capture_selective = 'CaptureSelective, http://schemas.evosnap.com/CWS/v2.0/Transactions/Rest'
    capture_all = 'CaptureAll, http://schemas.evosnap.com/CWS/v2.0/Transactions/Rest'
    return_by_id = 'ReturnById, http://schemas.evosnap.com/CWS/v2.0/Transactions/Rest'
    return_unlinked = 'ReturnTransaction,http://schemas.evosnap.com/CWS/v2.0/Transactions/Rest'


class TransactionType(Enum):
    bankcard = 'BankcardTransaction,http://schemas.evosnap.com/CWS/v2.0/Transactions/Bankcard'
    bankcard_profile = 'BankcardTransactionPro,http://schemas.evosnap.com/CWS/v2.0/Transactions/Bankcard/Pro'
    electronic_checking = 'ElectronicCheckingTransaction, http://schemas.evosnap.com/CWS/v2.0/Transactions/ElectronicChecking'
    stored_value = 'StoredValueTransaction, http://schemas.evosnap.com/CWS/v2.0/Transactions/StoredValueTransaction'
    encryption = 'EncryptionTransaction, http://schemas.evosnap.com/CWS/v2.0/Transactions/EncryptionTransaction'


class TransactionDataType(Enum):
    bankcard = 'BankcardTransactionDataPro, http://schemas.evosnap.com/CWS/v2.0/Transactions/Bankcard/Pro'
    electronic_checking = 'ElectronicCheckingTransactionData, http://schemas.evosnap.com/CWS/v2.0/Transactions/ElectronicChecking'


class CardType(Enum):
    american_express = 'AmericanExpress'
    carte_aurore = 'CarteAurore'
    cartes_bancaires = 'CartesBancaires'
    citibank_financial = 'CitibankFinancial'
    dankort = 'Dankort'
    diners_club = 'DinersClub'
    discover = 'Discover'
    electron = 'Electron'
    finax = 'Finax'
    jcb = 'JCB'
    kopkort = 'Kopkort'
    laser_card = 'LaserCard'
    maestro = 'Maestro'
    master_card = 'MasterCard'
    not_set = 'NotSet'
    private_label = 'PrivateLabel'
    revolution_card = 'RevolutionCard'
    solo = 'Solo'
    unbranded_atm = 'UnbrandedATM'
    visa = 'Visa'


class GoodsType(Enum):
    digital = 'DigitalGoods'
    not_set = 'NotSet'
    physical = 'PhysicalGoods'


class CustomerPresent(Enum):
    bill_payment = 'BillPayment'
    ecommerce = 'Ecommerce'
    mail_fax = 'MailFax'
    moto = 'MOTO'
    motocc = 'MOTOCC'
    not_set = 'NotSet'
    present = 'Present'
    suspicious = 'Suspicious'
    tel_aru = 'TelARU'
    transponder = 'Transponder'
    visa_unreadable = 'VisaCardPresentStripeUnreadable'
    visa_open_network = 'VisaOpenNetworkTransaction'
    voice_response = 'VoiceResponse'


class IndustryType(Enum):
    ecommerce = 'Ecommerce'
    moto = 'MOTO'
    not_set = 'NotSet'
    restaurant = 'Restaurant'
    retail = 'Retail'


class EntryMode(Enum):
    barcode = 'Barcode'
    chip_reliable = 'ChipReliable'
    chip_track_data_rfid = 'ChipTrackDataFromRFID'
    chip_unreliable = 'ChipUnreliable'
    contactless_chip_or_card = 'ContactlessMChipOrSmartCard'
    contactless_stripe = 'ContactlessStripe'
    keyed = 'Keyed'
    keyed_bad_read = 'KeyedBadMagRead'
    msrt_track_data_rfid = 'MSRTrackDataFromRFID'
    nfc_capable = 'NFCCapable'
    not_set = 'NotSet'
    ocr_reader = 'OCRReader'
    terminal_not_used = 'TerminalNotUsed'
    track2_data_msr = 'Track2DataFromMSR'
    track_data_msr = 'TrackDataFromMSR'
    vsc_capable = 'VSCCapable'


class AccountType(Enum):
    checking = 'CheckingAccount'
    not_set = 'NotSet'
    savings = 'SavingsAccount'


class CVDataProvided(Enum):
    not_available = 'CardholderStatesNotAvailable'
    bypass = 'DeliberatelyBypass'
    not_set = 'NotSet'
    provided = 'Provided'
    illegible = 'ValueIllegible'


class TokenIndicator(Enum):
    attempted_service_unsupported = 'AttemptedCardUnsupported'
    attempted_service_unavailable = 'AttemptedServiceUnavailable'
    not_set = 'NotSet'
    ucaf_with_data = 'UCAFWithData'
    ucaf_without_data = 'UCAFWithoutData'
    vpas = 'VPAS'


class ChargeType(Enum):
    beauty_shop = 'BeautyShop'
    conventional_fee = 'ConventionFee'
    gift_shop = 'GiftShop'
    golf_pro_shop = 'GolfProShop'
    health_spa = 'HealthSpa'
    lodging = 'Lodging'
    not_set = 'NotSet'
    restaurant = 'Restaurant'
    retail_other = 'RetailOther'
    tennis_pro_shop = 'TennisProShop'


class PINDebitUndoReason(Enum):
    cad_failure = 'CADFailure'
    customer_cancellation = 'CustomerCancellation'
    late_response = 'LateResponse'
    no_response = 'NoResponse'
    not_applicable = 'NotApplicable'
    not_set = 'NotSet'
    original_amount_incorrect = 'OriginalAmountIncorrect'
    partially_completed = 'PartiallyCompleted'
    response_incorrect = 'ResponseIncomplete'
    response_timeout = 'ResponseTimeout'
    suspect_malfunction = 'SuspectMalfunction'
    unable_to_deliver_response = 'UnableToDeliverResponse'
    unable_to_deliver_to_pos = 'UnableToDeliverToPOS'


class UndoReason(Enum):
    customer_cancellation = 'CustomerCancellation'
    not_set = 'NotSet'



from .transaction import Transaction
from .resubmit import Resubmit
from .transaction_data import TransactionData
from .transaction_tender_data import TransactionTenderData
from .undo import Undo
from .addendum import Addendum
from .adjust import Adjust
from .unmanaged import Unmanaged



from evosnap import constants
from evosnap.transactions.enum import UndoReason, PINDebitUndoReason


class Undo:
    def __init__(self, transaction_id, addendum=None, p_i_n_debit_reason=None, tender_data=None,
                 transaction_code=None, undo_reason=None, force_void=False):
        """
        Contains information for undoing (voiding) a transaction. This element is Required.
        :param activation: Contains information for activating an account. This is a required element.
        """
        self.__camelcase=constants.ALL_FIELDS
        self.__order=['$type', 'PINDebitReason', 'TenderData', 'TransactionId', 'Addendum', 'ForceVoid',
                      'TransactionCode', 'UndoReason']
        self.i_type='BankcardUndo,http://schemas.evosnap.com/CWS/v2.0/Transactions/Bankcard'
        self.addendum=addendum
        self.transaction_id=transaction_id
        self.p_i_n_debit_reason=p_i_n_debit_reason or PINDebitUndoReason.not_set
        self.tender_data=tender_data
        self.transaction_code=transaction_code or 'NotSet'
        self.undo_reason=undo_reason or UndoReason.customer_cancellation
        self.force_void=force_void



from evosnap import constants


class Addendum:
    def __init__(self, unmanaged=None):
        """
        The Addendum object is used to pass optional service specific information. This object is not required unless
        specifically directed to do so by your EVO Snap* Support team.
        :param activation: Contains information for activating an account. This is a required element.
        """
        self.__camelcase=constants.ALL_FIELDS
        self.unmanaged=unmanaged



from evosnap import constants


class PrimaryOwner:
    def __init__(self,**kwargs):
        self.__order = [
            'PrimaryOwnerPercentage', 'PrimaryOwnershipType', 'PrimaryOwnerFirstName', 'PrimaryOwnerLastName',
            'PrimaryOwnerAddress', 'PrimaryOwnerCity', 'PrimaryOwnerCounty', 'PrimaryOwnerPostalCode',
            'PrimaryOwnerCountry', 'PrimaryOwnerDOB', 'PrimaryOwnerAddress2',
            'PrimaryOwnerAddressBuildingNumber', 'PrimaryOwnerAddressFloorNumber',
            'PrimaryOwnerAddressDoorNumber', 'PrimaryOwnerHomePhonePrefix', 'PrimaryOwnerHomePhone',
            'PrimaryOwnerDocumentNumber', 'PrimaryOwnerIsOnPepList', 'PrimaryOwnerIsOnSancionList',
        ]
        self.__camelcase = constants.ALL_FIELDS

        self.primary_owner_percentage = kwargs.get('primary_owner_percentage')
        self.primary_ownership_type = kwargs.get('primary_ownership_type')
        self.primary_owner_first_name = kwargs.get('primary_owner_first_name')
        self.primary_owner_last_name = kwargs.get('primary_owner_last_name')
        self.primary_owner_address = kwargs.get('primary_owner_address')
        self.primary_owner_city = kwargs.get('primary_owner_city')
        self.primary_owner_county = kwargs.get('primary_owner_county')
        self.primary_owner_postal_code = kwargs.get('primary_owner_postal_code')
        self.primary_owner_country = kwargs.get('primary_owner_country')
        self.primary_owner_d_o_b = kwargs.get('primary_owner_d_o_b')
        self.primary_owner_address2 = kwargs.get('primary_owner_address2')
        self.primary_owner_address_building_number = kwargs.get('primary_owner_address_building_number')
        self.primary_owner_address_floor_number = kwargs.get('primary_owner_address_floor_number')
        self.primary_owner_address_door_number = kwargs.get('primary_owner_address_door_number')
        self.primary_owner_home_phone_prefix = kwargs.get('primary_owner_home_phone_prefix')
        self.primary_owner_home_phone = kwargs.get('primary_owner_home_phone')
        self.primary_owner_document_number = kwargs.get('primary_owner_document_number')
        self.primary_owner_is_on_pep_list = kwargs.get('primary_owner_is_on_pep_list')
        self.primary_owner_is_on_sancion_list = kwargs.get('primary_owner_is_on_sancion_list')

    @property
    def hash_str(self):
        required = [
            'primary_owner_percentage', 'primary_ownership_type', 'primary_owner_first_name', 'primary_owner_last_name',
            'primary_owner_address', 'primary_owner_city', 'primary_owner_county', 'primary_owner_postal_code',
            'primary_owner_country', 'primary_owner_d_o_b', 'primary_owner_address2',
            'primary_owner_address_building_number', 'primary_owner_address_floor_number',
            'primary_owner_address_door_number', 'primary_owner_home_phone_prefix', 'primary_owner_home_phone',
            'primary_owner_document_number', 'primary_owner_is_on_pep_list', 'primary_owner_is_on_sancion_list',
        ]
        return ''.join([str(getattr(self,f)).strip() for f in required if getattr(self,f) is not None])


import base64
import hashlib
import hmac
import json
import requests

from evosnap.response import Response
from evosnap.data_element_encoder import DataElementEncoder
from evosnap.exceptions import ApplicationRequestException


class MerchantApplicationRequest():
    def __init__(self, **kwargs):
        """
        Basic information about the check being processed. This element is Required.
        :param activation: Contains information for activating an account. This is a required element.
        """
        self.__order = [
            'HashValue',
            'IntegratorID', 'ReferenceID', 'OriginIPAddress', 'CultureCode', 'Email', 'URL',
            'ApplicationType', 'CompanyName', 'RegisteredCompanyName', 'ContactFirstName', 'ContactLastName',
            'ContactPostalCode', 'PhysicalAddress', 'PhysicalCity', 'PhysicalCounty', 'PhysicalPostalCode',
            'PhysicalCountry', 'PhysicalPhone', 'BusinessStructureTypeId', 'MerchantCategoryCode',
            'CompanyObjective', 'VATID', 'BankName', 'BankAccountHolderName', 'BankIBAN',
            'BankBICSWIFT', 'InvoiceBankName', 'InvoiceBankAccountOwner', 'InvoiceBankIBAN',
            'InvoiceBankAddress1', 'InvoiceBankCity', 'ForecastMaxTicketSizeAmount', 'ForecastAllVolume',
            'ForecastAveTicketSizeAmount', 'ForecastRefundPercentage', 'PSPaidInAdvanceTF',
            'PSPaidInAdvanceAvgAnnualPercentage', 'PSPaidInAdvanceAvgDeliveryDays', 'PCIHasSAQ', 'PCIThirdPartyRemotes',
            'PCICardDataStored', 'PCIHadDataCompromise', 'NoVat', 'PrimaryOwnerArray',
            'ProductPackageID', 'OtherServicesMOTO', 'OtherServicesSettlement',
            'OtherServicesAMEXRouting', 'OtherServicesSnapEcommerce', 'OtherServicesChargebacks',
            'OtherServicesRecurringPayments', 'OtherServicesChinaUnionPay', 'OtherServices3DSecure',
            'OtherServicesCashback', 'OtherServicesPremiumService', 'OtherServicesDCC',
            'OtherServicesReceiptLogo', 'OtherServicesTipp', 'OtherServicesMonths',
            'OtherServicesMonthsDeclaration', 'OtherServicesCVV', 'EcommGateway',
            'posDeploymentArray', 'InvoiceBankAddress2', 'InvoiceBankAddressFloorNumber',
            'InvoiceBankAddressDoorNumber', 'InvoiceBankAddressBuildingNumber', 'InvoiceBankZip',
            'DocumentationVersion', 'PricingComponentSettingArray', 'IsOnPepList', 'IsOnSanctionList',
            'EcommSolutionType', 'EcommShoppingCart', 'EcommIPAddress', 'TechnicalContactFirstName',
            'TechnicalContactLastName', 'TechnicalContactEmail', 'TechnicalContactPhoneNumber',
            'EcommAlternativePaymentMethod', 'PaymentMethodOtherName', 'PaymentUsername', 'PaymentPassword',
            'PaymentSignature', 'PaymentLogoURL', 'CurrencyCode',
        ]
        self.__camelcase = [
            'hash_value',
            'integrator_i_d', 'reference_i_d', 'origin_i_p_address', 'culture_code', 'email', 'u_r_l',
            'application_type', 'company_name', 'registered_company_name', 'contact_first_name', 'contact_last_name',
            'contact_postal_code', 'physical_address', 'physical_city', 'physical_county', 'physical_postal_code',
            'physical_country', 'physical_phone', 'business_structure_type_id', 'merchant_category_code',
            'company_objective', 'v_a_t_i_d', 'bank_name', 'bank_account_holder_name', 'bank_i_b_a_n',
            'bank_b_i_c_s_w_i_f_t', 'invoice_bank_name', 'invoice_bank_account_owner', 'invoice_bank_i_b_a_n',
            'invoice_bank_address1', 'invoice_bank_city', 'forecast_max_ticket_size_amount',
            'forecast_all_volume', 'forecast_ave_ticket_size_amount', 'forecast_refund_percentage',
            'p_s_paid_in_advance_t_f', 'p_s_paid_in_advance_avg_annual_percentage',
            'p_s_paid_in_advance_avg_delivery_days', 'p_c_i_has_s_a_q', 'p_c_i_third_party_remotes',
            'p_c_i_card_data_stored', 'p_c_i_had_data_compromise', 'no_vat', 'primary_owner_array',
            'product_package_i_d', 'other_services_m_o_t_o', 'other_services_settlement',
            'other_services_a_m_e_x_routing', 'other_services_snap_ecommerce', 'other_services_chargebacks',
            'other_services_recurring_payments', 'other_services_china_union_pay', 'other_services_3_d_secure',
            'other_services_cashback', 'other_services_premium_service', 'other_services_d_c_c',
            'other_services_receipt_logo', 'other_services_tipp', 'other_services_months',
            'other_services_months_declaration', 'other_services_c_v_v', 'ecomm_gateway', 'invoice_bank_address2',
            'invoice_bank_address_floor_number', 'invoice_bank_address_door_number',
            'invoice_bank_address_building_number', 'invoice_bank_zip', 'documentation_version',
            'pricing_component_setting_array', 'is_on_pep_list', 'is_on_sanction_list',
            'ecomm_solution_type', 'ecomm_shopping_cart', 'ecomm_i_p_address', 'technical_contact_first_name',
            'technical_contact_last_name', 'technical_contact_email', 'technical_contact_phone_number',
            'ecomm_alternative_payment_method', 'payment_method_other_name', 'payment_username', 'payment_password',
            'payment_signature', 'payment_logo_u_r_l', 'currency_code',
        ]
        self.__lower_camelcase = ['pos_deployment_array']
        self.__exclude = ['ssl_verification', 'request_method', 'mas_baseurl', 'secret']

        self.ssl_verification = kwargs.get('ssl_verification', True)
        self.request_method = 'POST'

        self.mas_baseurl = kwargs.get('mas_baseurl', 'https://api.certification.boipa.com/MAS/api/v1/Submit')
        self.secret = kwargs.get('secret')

        # REQUIRED
        self.integrator_i_d = kwargs.get('integrator_i_d')
        self.reference_i_d = kwargs.get('reference_i_d')
        self.origin_i_p_address = kwargs.get('origin_i_p_address')
        self.culture_code = kwargs.get('culture_code')
        self.email = kwargs.get('email')
        self.u_r_l = kwargs.get('u_r_l')
        self.application_type = kwargs.get('application_type')
        self.company_name = kwargs.get('company_name')
        self.registered_company_name = kwargs.get('registered_company_name')
        self.contact_first_name = kwargs.get('contact_first_name')
        self.contact_last_name = kwargs.get('contact_last_name')
        self.contact_postal_code = kwargs.get('contact_postal_code')
        self.physical_address = kwargs.get('physical_address')
        self.physical_city = kwargs.get('physical_city')
        self.physical_county = kwargs.get('physical_county')
        self.physical_postal_code = kwargs.get('physical_postal_code')
        self.physical_country = kwargs.get('physical_country')
        self.physical_phone = kwargs.get('physical_phone')
        self.business_structure_type_id = kwargs.get('business_structure_type_id')
        self.merchant_category_code = kwargs.get('merchant_category_code')
        self.company_objective = kwargs.get('company_objective')
        self.v_a_t_i_d = kwargs.get('v_a_t_i_d')
        self.bank_name = kwargs.get('bank_name')
        self.bank_account_holder_name = kwargs.get('bank_account_holder_name')
        self.bank_i_b_a_n = kwargs.get('bank_i_b_a_n')
        self.bank_b_i_c_s_w_i_f_t = kwargs.get('bank_b_i_c_s_w_i_f_t')
        self.invoice_bank_name = kwargs.get('invoice_bank_name')
        self.invoice_bank_account_owner = kwargs.get('invoice_bank_account_owner')
        self.invoice_bank_i_b_a_n = kwargs.get('invoice_bank_i_b_a_n')
        self.invoice_bank_address1 = kwargs.get('invoice_bank_address1')
        self.invoice_bank_city = kwargs.get('invoice_bank_city')
        self.invoice_bank_country = kwargs.get('invoice_bank_country')
        self.forecast_max_ticket_size_amount = kwargs.get('forecast_max_ticket_size_amount')
        self.forecast_all_volume = kwargs.get('forecast_all_volume')
        self.forecast_ave_ticket_size_amount = kwargs.get('forecast_ave_ticket_size_amount')
        self.forecast_refund_percentage = kwargs.get('forecast_refund_percentage')
        self.p_s_paid_in_advance_t_f = kwargs.get('p_s_paid_in_advance_t_f')
        self.p_s_paid_in_advance_avg_annual_percentage = kwargs.get('p_s_paid_in_advance_avg_annual_percentage')
        self.p_s_paid_in_advance_avg_delivery_days = kwargs.get('p_s_paid_in_advance_avg_delivery_days')
        self.p_c_i_has_s_a_q = kwargs.get('p_c_i_has_s_a_q')
        self.p_c_i_third_party_remotes = kwargs.get('p_c_i_third_party_remotes')
        self.p_c_i_card_data_stored = kwargs.get('p_c_i_card_data_stored')
        self.p_c_i_had_data_compromise = kwargs.get('p_c_i_had_data_compromise')
        self.no_vat = kwargs.get('no_vat')
        self.primary_owner_array = kwargs.get('primary_owner_array')
        self.product_package_i_d = kwargs.get('product_package_i_d')
        self.other_services_m_o_t_o = kwargs.get('other_services_m_o_t_o')
        self.other_services_settlement = kwargs.get('other_services_settlement')
        self.other_services_a_m_e_x_routing = kwargs.get('other_services_a_m_e_x_routing')
        self.other_services_snap_ecommerce = kwargs.get('other_services_snap_ecommerce')
        self.other_services_chargebacks = kwargs.get('other_services_chargebacks')
        self.other_services_recurring_payments = kwargs.get('other_services_recurring_payments')
        self.other_services_china_union_pay = kwargs.get('other_services_china_union_pay')
        self.other_services_3_d_secure = kwargs.get('other_services_3_d_secure')
        self.other_services_cashback = kwargs.get('other_services_cashback')
        self.other_services_premium_service = kwargs.get('other_services_premium_service')
        self.other_services_d_c_c = kwargs.get('other_services_d_c_c')
        self.other_services_receipt_logo = kwargs.get('other_services_receipt_logo')
        self.other_services_tipp = kwargs.get('other_services_tipp')
        self.other_services_months = kwargs.get('other_services_months')
        self.other_services_months_declaration = kwargs.get('other_services_months_declaration')
        self.other_services_c_v_v = kwargs.get('other_services_c_v_v')
        self.ecomm_gateway = kwargs.get('ecomm_gateway')
        self.pos_deployment_array = kwargs.get('pos_deployment_array')
        self.invoice_bank_address2 = kwargs.get('invoice_bank_address2')
        self.invoice_bank_address_floor_number = kwargs.get('invoice_bank_address_floor_number')
        self.invoice_bank_address_door_number = kwargs.get('invoice_bank_address_door_number')
        self.invoice_bank_address_building_number = kwargs.get('invoice_bank_address_building_number')
        self.invoice_bank_zip = kwargs.get('invoice_bank_zip')
        self.documentation_version = kwargs.get('documentation_version')

        # ECOMMERCE

        self.pricing_component_setting_array = kwargs.get('pricing_component_setting_array')
        self.is_on_pep_list = kwargs.get('is_on_pep_list')
        self.is_on_sanction_list = kwargs.get('is_on_sanction_list')
        self.ecomm_solution_type = kwargs.get('ecomm_solution_type')
        self.ecomm_shopping_cart = kwargs.get('ecomm_shopping_cart')
        self.ecomm_i_p_address = kwargs.get('ecomm_i_p_address')
        self.technical_contact_first_name = kwargs.get('technical_contact_first_name')
        self.technical_contact_last_name = kwargs.get('technical_contact_last_name')
        self.technical_contact_email = kwargs.get('technical_contact_email')
        self.technical_contact_phone_number = kwargs.get('technical_contact_phone_number')
        self.ecomm_alternative_payment_method = kwargs.get('ecomm_alternative_payment_method')
        self.payment_method_other_name = kwargs.get('payment_method_other_name')
        self.payment_username = kwargs.get('payment_username')
        self.payment_password = kwargs.get('payment_password')
        self.payment_signature = kwargs.get('payment_signature')
        self.payment_logo_u_r_l = kwargs.get('payment_logo_u_r_l')
        self.currency_code = kwargs.get('currency_code')

        self.hash_value = self.hash

    @property
    def request_json(self):
        return json.dumps(self, cls=DataElementEncoder)

    @property
    def pretty_json(self):
        return json.dumps(self, cls=DataElementEncoder, indent=4)

    @property
    def primary_owner_array_hash(self):
        return ''.join([f.hash_str for f in self.primary_owner_array or []])

    @property
    def pos_deployment_array_hash(self):
        return ''.join([f.hash_str for f in self.pos_deployment_array or []])

    @property
    def pricing_component_setting_array_hash(self):
        return ''.join([f.hash_str for f in self.pricing_component_setting_array or []])

    @property
    def hash_str(self):
        required = [
            'integrator_i_d', 'reference_i_d', 'origin_i_p_address', 'culture_code', 'email', 'u_r_l',
            'application_type', 'company_name', 'registered_company_name', 'contact_first_name', 'contact_last_name',
            'contact_postal_code', 'physical_address', 'physical_city', 'physical_county', 'physical_postal_code',
            'physical_country', 'physical_phone', 'business_structure_type_id', 'merchant_category_code',
            'company_objective', 'v_a_t_i_d', 'bank_name', 'bank_account_holder_name', 'bank_i_b_a_n',
            'bank_b_i_c_s_w_i_f_t', 'invoice_bank_name', 'invoice_bank_account_owner', 'invoice_bank_i_b_a_n',
            'invoice_bank_address1', 'invoice_bank_city', 'invoice_bank_country', 'forecast_max_ticket_size_amount',
            'forecast_all_volume', 'forecast_ave_ticket_size_amount', 'forecast_refund_percentage',
            'p_s_paid_in_advance_t_f', 'p_s_paid_in_advance_avg_annual_percentage',
            'p_s_paid_in_advance_avg_delivery_days', 'p_c_i_has_s_a_q', 'p_c_i_third_party_remotes',
            'p_c_i_card_data_stored', 'p_c_i_had_data_compromise', 'no_vat', 'primary_owner_array_hash',
            'product_package_i_d', 'other_services_m_o_t_o', 'other_services_settlement',
            'other_services_a_m_e_x_routing', 'other_services_snap_ecommerce', 'other_services_chargebacks',
            'other_services_recurring_payments', 'other_services_china_union_pay', 'other_services_3_d_secure',
            'other_services_cashback', 'other_services_premium_service', 'other_services_d_c_c',
            'other_services_receipt_logo', 'other_services_tipp', 'other_services_months',
            'other_services_months_declaration', 'other_services_c_v_v', 'ecomm_gateway',
            'pos_deployment_array_hash', 'invoice_bank_address2',
            'invoice_bank_address_floor_number', 'invoice_bank_address_door_number',
            'invoice_bank_address_building_number', 'invoice_bank_zip', 'documentation_version',
            'pricing_component_setting_array_hash', 'is_on_pep_list', 'is_on_sanction_list',
            'ecomm_solution_type', 'ecomm_shopping_cart', 'ecomm_i_p_address', 'technical_contact_first_name',
            'technical_contact_last_name', 'technical_contact_email', 'technical_contact_phone_number',
            'ecomm_alternative_payment_method', 'payment_method_other_name', 'payment_username', 'payment_password',
            'payment_signature', 'payment_logo_u_r_l', 'currency_code',
        ]

        return ''.join([str(getattr(self,f)).strip() for f in required if getattr(self,f) is not None])


    @property
    def hash(self):
        h = hmac.new(
            bytes.fromhex(self.secret),
            msg=self.hash_str.encode('utf-8'),
            digestmod=hashlib.sha256
        )
        dig = h.digest()
        return base64.b64encode(dig).decode('utf-8')


    @property
    def request_endpoint(self):
        return self.mas_baseurl

    def send(self, test_request=None):
        response = requests.request(
            method=self.request_method,
            url=self.request_endpoint,
            headers={'Content-Type': 'application/json', 'Accept': 'application/json'},
            data=self.request_json if not test_request else json.dumps(test_request),
            verify=self.ssl_verification
        )
        if response.status_code!=200 and response.status_code!=201 and response.status_code!=400:
            try:
                msg = str(response.status_code)+' '+Response(response.text).to_pretty_json()
                raise ApplicationRequestException(msg)
            except json.JSONDecodeError:
                msg = str(response.status_code) + ' ' + response.text
                raise ApplicationRequestException(msg)
        return Response(response.text)


LIVE=False
PROXY=False
PROXY_HOST='127.0.0.1'
PROXY_PORT=8888
TRUST_ALL=False

ALL_FIELDS='__ALL__'

SESSION_TOKEN_TIME = 1740

CWS_SIS_BASEURL = 'https://api.cip.goevo.com/2.1.27/REST/SIS.svc'
CWS_TPS_BASEURL = 'https://api.cip.goevo.com/2.1.27/REST/TPS.svc'
CWS_TMS_BASEURL = 'https://api.cip.goevo.com/2.1.27/REST/DataServices/TMS.svc'
CWS_UAT_SIS_BASEURL = 'https://api.cipcert.goevo.com/2.1.27/REST/SIS.svc'
CWS_UAT_TPS_BASEURL = 'https://api.cipcert.goevo.com/2.1.27/REST/TPS.svc'
CWS_UAT_TMS_BASEURL = 'https://api.cipcert.goevo.com/2.1.27/REST/DataServices/TMS.svc'



from evosnap import constants


class PricingComponentSetting:
    def __init__(self,**kwargs):
        self.__order = [
            'ComponentName', 'Option', 'Value',
        ]
        self.__lower_camelcase = constants.ALL_FIELDS

        self.component_name = kwargs.get('component_name')
        self.option = kwargs.get('option')
        self.value = kwargs.get('value')

    @property
    def hash_str(self):
        required = [
            'component_name', 'option', 'value'
        ]
        return ''.join([str(getattr(self,f)).strip() for f in required if getattr(self,f) is not None])



from evosnap.data_element_encoder import DataElementEncoder
from evosnap.request import Request
from evosnap.application_request import MerchantApplicationRequest
from evosnap.response import Response
from evosnap.transactions.base.transaction_tender_data import TransactionTenderData
from evosnap.transactions.base.transaction import Transaction
from evosnap.transactions.base.undo import Undo
from evosnap.transactions.base.resubmit import Resubmit
from evosnap.transactions.base.addendum import Addendum
from evosnap.transactions.base.adjust import Adjust
from evosnap.transactions.base.transaction_data import TransactionData
from evosnap.transactions.base.transaction_customer_data import TransactionCustomerData
from evosnap.transactions.base.customer_info import CustomerInfo
from evosnap.transactions.base.unmanaged import Unmanaged
from evosnap.transactions.bankcard.capture import Capture
from evosnap.transactions.bankcard.card_data import CardData
from evosnap.transactions.bankcard.card_security_data import CardSecurityData
from evosnap.transactions.bankcard.ecommerce_security_data import EcommerceSecurityData
from evosnap.transactions.bankcard.international_avs_data import InternationalAVSData
from evosnap.transactions.bankcard.international_avs_override import InternationalAVSOverride
from evosnap.transactions.bankcard.internet_transaction_data import InternetTransactionData
from evosnap.transactions.bankcard.return_by_id import ReturnById
from evosnap.transactions.enum import RequestType, TransactionType, TransactionDataType, CardType, GoodsType, \
    CustomerPresent, IndustryType, EntryMode, AccountType, CVDataProvided, TokenIndicator, ChargeType, \
    PINDebitUndoReason, UndoReason
from evosnap.merchant_applications.pos_deployment_location import POSDeploymentLocation
from evosnap.merchant_applications.pos_device import POSDevice
from evosnap.merchant_applications.primary_owner import PrimaryOwner
from .exceptions import TransactionRequestException, AppProfileException, SignOnException, ApplicationRequestException



from evosnap import constants


class InternationalAVSOverride:
    def __init__(self, skip_a_v_s=False, ignore_a_v_s=False, a_v_s_reject_codes=None):
        """
        Cardholder address data for Address Verification System (AVS). This element is Optional.
        :param activation: Contains information for activating an account. This is a required element.
        """
        self.__camelcase=constants.ALL_FIELDS
        self.__order = ['SkipAVS', 'IgnoreAVS', 'AVSRejectCodes']
        self.skip_a_v_s=skip_a_v_s
        self.ignore_a_v_s=ignore_a_v_s
        self.a_v_s_reject_codes=a_v_s_reject_codes



from .base import Transaction, TransactionData, TransactionTenderData, Unmanaged, Undo, Addendum, Adjust
from .bankcard import CardData, CardSecurityData, EcommerceSecurityData, InternationalAVSData, ReturnById, Capture
from .bankcard import InternetTransactionData
from .enum import RequestType, TransactionType, TransactionDataType, CardType, GoodsType, CustomerPresent, \
    IndustryType, EntryMode, AccountType, CVDataProvided, TokenIndicator, ChargeType, PINDebitUndoReason, UndoReason



from evosnap import constants


class CustomerInfo:
    def __init__(self, address=None, business_name=None, email=None,
                 fax=None, name=None, phone=None):
        """
        Contains information about the customer. This element is required by some service providers. If this value is
        set on the transaction and not required by the service provider, the data is not present on the transaction.
        :param activation: Contains information for activating an account. This is a required element.
        """
        self.__camelcase=constants.ALL_FIELDS
        self.__order=['Address', 'BusinessName', 'Email', 'Fax', 'Name', 'Phone']
        self.address=address
        self.business_name=business_name
        self.email=email
        self.fax=fax
        self.name=name
        self.phone=phone



import json
from json import JSONDecodeError


class Response:
    def __init__(self, data=None):
        """
        Defines a Response object
        :param data: a string containing xml data
        """
        from evosnap import TransactionRequestException
        if data:
            if isinstance(data,str):
                try:
                    self._json = json.loads(data)
                except JSONDecodeError:
                    raise TransactionRequestException(data)
            elif isinstance(data, dict):
                self._json = data
            else:
                raise TypeError('Data must be a dictionary or string')
        else:
            self._json = {}

    def __getattr__(self, item):
        """
        Search unidentified attributes inside the structure inside the _json attribute.
        :param item: the name of the requested attribute
        :return: the content of the first attribute named item in the structure
        """
        try:
            node = item.split('__')
            element = self._json
            for n in node[:-1]:
                element = element.get(n, {})
            return element.get(node[-1])
        except AttributeError:
            raise AttributeError('{} not found in response.'.format(item))

    def to_json_string(self):
        """
        Turns the data contained by this object into a json string.
        :return: a json string with the data contained by this object.
        """
        return json.dumps(self._json)

    def to_pretty_json(self):
        """
        Turns the data contained by this object into a json string.
        :return: a json string with the data contained by this object.
        """
        return json.dumps(self._json, indent=4, sort_keys=True)



from evosnap import constants


class InternationalAVSData:
    def __init__(self, cardholder_name=None, street=None, city=None, country=None, email=None, phone=None, postal_code=None,
                 state_province=None, house_number=None):
        """
        Cardholder address data for Address Verification System (AVS). This element is Optional.
        :param activation: Contains information for activating an account. This is a required element.
        """
        self.__camelcase=constants.ALL_FIELDS
        self.__order = ['CardholderName', 'Street', 'City', 'Country', 'Email', 'Phone', 'PostalCode',
                 'StateProvince', 'HouseNumber']
        self.cardholder_name=cardholder_name
        self.street=street
        self.city=city
        self.country=country
        self.email=email
        self.phone=phone
        self.postal_code=postal_code
        self.state_province=state_province
        self.house_number=house_number



from evosnap import constants
from evosnap.transactions.enum import TransactionType


class Transaction:
    def __init__(self, transaction_type, tender_data, transaction_data, addendum=None, customer_data=None, reporting_data=None,
                 application_configuration_data=None):
        """
        Contains all information required to send a transaction to a service provider. This element is Required.
        :param activation: Contains information for activating an account. This is a required element.
        """
        self.__camelcase=constants.ALL_FIELDS
        self.__order=['$type','Addendum', 'CustomerData', 'ReportingData', 'TenderData', 'TransactionData',
                          'ApplicationConfigurationData']

        if not isinstance(transaction_type, TransactionType):
            raise TypeError('transaction_type must be an instance of TransactionType')

        self.i_type=transaction_type
        self.addendum=addendum
        self.customer_data=customer_data
        self.reporting_data=reporting_data
        self.tender_data=tender_data
        self.transaction_data=transaction_data
        self.application_configuration_data=application_configuration_data



import datetime

from evosnap import constants
from evosnap.transactions.enum import ChargeType


class Capture:
    def __init__(self, transaction_id, amount=None, tip_amount=None, addendum=None):
        """
        Cardholder address data for Address Verification System (AVS). This element is Optional.
        :param activation: Contains information for activating an account. This is a required element.
        """
        self.__camelcase=constants.ALL_FIELDS
        self.__order=['$type', 'Amount', 'TransactionId', 'TipAmount', 'Addendum']
        self.i_type='BankcardCapture,http://schemas.evosnap.com/CWS/v2.0/Transactions/Bankcard'
        self.amount=amount or '0.00'
        self.addendum=addendum
        self.tip_amount=tip_amount or '0.00'
        self.transaction_id=transaction_id


class SignOnException(Exception):
    pass


class AppProfileException(Exception):
    pass


class TransactionRequestException(Exception):
    pass


class ApplicationRequestException(Exception):
    pass



from .card_data import CardData
from .capture import Capture
from .internet_transaction_data import InternetTransactionData
from .international_avs_data import InternationalAVSData
from .card_security_data import CardSecurityData
from .ecommerce_security_data import EcommerceSecurityData
from .return_by_id import ReturnById
from .internet_transaction_data import InternetTransactionData


from evosnap import constants


class EcommerceSecurityData:
    def __init__(self, token_data=None, token_indicator=None, x_i_d=None):
        """
        Ecommerce security elements. This element is Conditional, optional for Ecommerce transactions,
        otherwise not present.
        :param activation: Contains information for activating an account. This is a required element.
        """
        self.__camelcase=constants.ALL_FIELDS
        self.__order=['TokenData', 'TokenIndicator', 'XID']
        self.token_data=token_data
        self.token_indicator=token_indicator
        self.x_i_d=x_i_d



from distutils.core import setup

setup(
    name='evosnap',
    packages=[
        'evosnap',
        'evosnap.service_information',
        'evosnap.transactions',
        'evosnap.transactions.base',
        'evosnap.transactions.bankcard',
        'evosnap.merchant_applications',
    ],
    version='0.3.6',
    description='Python 3 package that integrates with EVO Snap* Commerce Web Services API',
    author='Javier Ros Honduvilla',
    author_email='javier.ros.honduvilla@gmail.com',
    url='https://github.com/Zertifica/evosnap',
    download_url='https://github.com/Zertifica/evosnap/tarball/0.1',
    keywords=['popular payments', 'tpv', 'snap'],
    classifiers=[],
)



import datetime

from evosnap import constants
from evosnap.transactions.bankcard import InternetTransactionData
from evosnap.transactions.enum import TransactionDataType, IndustryType, CustomerPresent, EntryMode, AccountType, \
    GoodsType


class TransactionData:
    def __init__(self, amount, currency_code, campaign_id=None, reference=None,
                 transaction_date_time=None, customer_present:CustomerPresent=CustomerPresent.not_set,
                 entry_mode:EntryMode=EntryMode.not_set, order_number=None, signature_captured=False,
                 account_type:AccountType=AccountType.not_set, alternative_merchant_data=None, approval_code=None,
                 batch_assignment=None, batch_id=None, cash_back_amount=None, employee_id=None, fee_amount=None,
                 goods_type:GoodsType=GoodsType.not_set, industry_type:IndustryType=IndustryType.not_set,
                 internet_transaction_data:InternetTransactionData=None, invoice_number=None, is_partial_shipment=None,
                 is_quasi_cash=None, lane_id=None, partial_approval_capable=None, score_threshold=None,
                 terminal_id=None, tip_amount=None, transaction_code=None,
                 transaction_data_type:TransactionDataType=None, is_3_d_secure=None):
        """
        Base class object containing information for a specific transaction. This element is Required.
        :param activation: Contains information for activating an account. This is a required element.
        """
        self.__camelcase=constants.ALL_FIELDS
        self.__order=[]

        if transaction_data_type:
            self.i_type = transaction_data_type
            self.__order.append('$type')
        self.amount=amount
        self.__order.append('Amount')
        if campaign_id:
            self.campaign_id=campaign_id
            self.__order.append('CampaignId')
        self.currency_code=currency_code
        self.__order.append('CurrencyCode')
        if reference:
            self.reference=reference
            self.__order.append('Reference')
        self.transaction_date_time=transaction_date_time or datetime.datetime.now().isoformat()
        self.__order.append('TransactionDateTime')
        if transaction_code:
            self.transaction_code=transaction_code
            self.__order.append('TransactionCode')
        self.account_type=account_type
        self.__order.append('AccountType')
        if alternative_merchant_data:
            self.alternative_merchant_data=alternative_merchant_data
            self.__order.append('AlternativeMerchantData')
        self.approval_code=approval_code
        self.__order.append('ApprovalCode')
        if batch_assignment:
            self.batch_assignment=batch_assignment
            self.__order.append('BatchAssignment')
        if batch_id:
            self.batch_id=batch_id
            self.__order.append('BatchId')
        self.cash_back_amount=cash_back_amount if cash_back_amount else '0.00'
        self.__order.append('CashBackAmount')
        self.customer_present=customer_present
        self.__order.append('CustomerPresent')
        self.employee_id=employee_id
        self.__order.append('EmployeeId')
        self.entry_mode=entry_mode
        self.__order.append('EntryMode')
        self.fee_amount=fee_amount if fee_amount else '0.00'
        self.__order.append('FeeAmount')
        self.goods_type=goods_type
        self.__order.append('GoodsType')
        self.industry_type=industry_type
        self.__order.append('IndustryType')
        self.internet_transaction_data=internet_transaction_data
        self.__order.append('InternetTransactionData')
        self.invoice_number=invoice_number
        if is_3_d_secure:
            self.is_3_d_secure=is_3_d_secure
            self.__order.append('Is3DSecure')
        self.__order.append('InvoiceNumber')
        if is_partial_shipment:
            self.is_partial_shipment=is_partial_shipment
            self.__order.append('IsPartialShipment')
        if is_quasi_cash:
            self.is_quasi_cash=is_quasi_cash
            self.__order.append('IsQuasiCash')
        if lane_id:
            self.lane_id=lane_id
            self.__order.append('LaneId')
        self.order_number=order_number
        self.__order.append('OrderNumber')
        if partial_approval_capable:
            self.partial_approval_capable=partial_approval_capable
            self.__order.append('PartialApprovalCapable')
        if score_threshold:
            self.score_threshold=score_threshold
            self.__order.append('ScoreThreshold')
        self.signature_captured=signature_captured
        self.__order.append('SignatureCaptured')
        if terminal_id:
            self.terminal_id=terminal_id
            self.__order.append('TerminalId')
        self.tip_amount=tip_amount if tip_amount else '0.00'
        self.__order.append('TipAmount')



from evosnap import constants
from evosnap.transactions.enum import UndoReason, PINDebitUndoReason


class Resubmit:
    def __init__(self, transaction_id, addendum=None, payment_authorization_response=None, c_v_v=None):
        """
        Contains information for resubmitting a transaction. This element is Required.
        :param activation: Contains information for activating an account. This is a required element.
        """
        self.__camelcase=constants.ALL_FIELDS
        self.__order=['$type', 'PaymentAuthorizationResponse', 'Addendum', 'CVV', 'TransactionId']
        self.i_type='Resubmit3DSecure, http://schemas.evosnap.com/CWS/v2.0/Transactions/Bankcard'
        self.payment_authorization_response = payment_authorization_response
        self.addendum=addendum
        self.c_v_v= c_v_v
        self.transaction_id=transaction_id



import datetime

from evosnap import constants


class ReturnById:
    def __init__(self, transaction_id, amount=None, addendum=None, transaction_date_time=None):
        """
        Contains information for returning Bankcard transactions (Credit and PIN Debit).
        :param activation: Contains information for activating an account. This is a required element.
        """
        self.__camelcase=constants.ALL_FIELDS
        self.__order=['$type', 'TransactionId', 'Amount', 'Addendum', 'TransactionDateTime']
        self.i_type='BankcardReturn,http://schemas.evosnap.com/CWS/v2.0/Transactions/Bankcard'
        self.amount=amount
        self.addendum=addendum
        self.transaction_date_time=transaction_date_time or datetime.datetime.now().isoformat()
        # self.transaction_data=transaction_data
        self.transaction_id=transaction_id



from evosnap import constants


class InternetTransactionData:
    def __init__(self, ip_address=None, session_id=None):
        """
        Contains information about the internet connection. This element is required by some service providers.
        If this value is set on the transaction and not required by the service provider, the data is not
        present on the transaction.
        :param activation: Contains information for activating an account. This is a required element.
        """
        self.__camelcase=constants.ALL_FIELDS
        self.__order=['IpAddress', 'SessionId']
        self.ip_address=ip_address
        self.session_id=session_id
